import { world, EquipmentSlot, system, Player } from "@minecraft/server";

// Detecta quando um jogador morre
world.afterEvents.entityDie.subscribe(event => {
    const entity = event.deadEntity;

    if (entity instanceof Player) {
        const player = entity;
        if (player.getGameMode() !== "creative") {
            const equipment = player.getComponent("minecraft:equippable");
            const headEquipment = equipment?.getEquipment(EquipmentSlot.Head);

            if (headEquipment && headEquipment.typeId === "mc:hades_mantle") {
                // Dar o item "mc:hades_portal_orb" ao jogador
                player.runCommandAsync(`give @s mc:hades_portal_orb 1`);

                // Criar um identificador único para o jogador (UUID)
                const playerId = player.id;

                // Criar o portal preto no local da morte do jogador
                const position = player.location;
                const portal = player.dimension.spawnEntity("mc:hades_portal_black", position);

                // Adicionar uma tag única ao portal preto
                portal.addTag(`portal_black_${playerId}`);
            }
        }
    }
});

world.beforeEvents.itemUseOn.subscribe(async ({ itemStack: item, source: player, block }) => {
    if (item && item.typeId == "mc:hades_portal_orb" && player.getGameMode() !== "creative") {
        const position = block.location;  // Usa a localização do bloco onde o item foi usado
        const playerId = player.id;
        const equipment = player.getComponent("equippable");
        const selectedItem = equipment.getEquipment("Mainhand");

        // Comando para criar a entidade do portal laranja
        const command = `summon mc:hades_portal_orange ${position.x} ${position.y + 1} ${position.z}`;

        // Executa o comando para gerar a entidade
        await player.runCommandAsync(command);

        // Encontre a entidade criada (isso pode depender do seu fluxo de script)
        const portalOrange = player.dimension.getEntities({ location: position, maxDistance: 10 })[0];

        if (portalOrange) {
            // Adiciona a tag ao portal
            portalOrange.addTag(`portal_orange_${playerId}`);

            console.warn(`Portal laranja criado em ${position.x}, ${position.y}, ${position.z}`);
        }
        if (selectedItem.amount === 1 && item.typeId == "mc:hades_portal_orb") {
                equipment.setEquipment("Mainhand", undefined);
              }
    }
});


system.afterEvents.scriptEventReceive.subscribe((data) => {
    if (data.id != "mc:portal_orange_teleport") return;

    const portalOrange = data.sourceEntity; // Quem ativou o evento (deve ser o portal laranja)

    if (!portalOrange) {
        console.warn("O evento foi acionado, mas a entidade de origem é indefinida.");
        return;
    }

    const dimension = portalOrange.dimension;

    // Pegar o jogador mais próximo do portal laranja
    const players = dimension.getPlayers({ location: portalOrange.location, maxDistance: 2 }); // Raio pequeno para garantir que pegamos apenas o jogador certo
    const player = players[0];

    if (!player) {
        console.warn("Nenhum jogador encontrado próximo ao portal laranja.");
        return;
    }

    const playerId = player.id;

    console.warn(`Evento acionado pelo portal laranja. Encontrado jogador ${player.name} (ID: ${playerId}).`);

    // Procurar pela entidade do portal preto que tem a tag associada ao playerId
    const ent = dimension.getEntities({ tags: [`portal_black_${playerId}`] })[0];

    if (!ent) {
        console.warn(`Nenhum portal preto encontrado para o jogador ${player.name}.`);
        return;
    }

    console.warn(`Portal preto encontrado! Teletransportando ${player.name} para ${ent.location.x}, ${ent.location.y}, ${ent.location.z}.`);

    // Teletransporta o jogador para a localização do portal preto
    player.teleport(ent.location);

    // Remove a tag temporária do portal preto, se necessário
    player.runCommandAsync('event entity @e[type=mc:hades_portal_black] remove_tag_timer');

    // Envia uma mensagem na actionbar para todos os jogadores
    player.runCommandAsync('titleraw @a actionbar {"rawtext":[{"translate":"function.horb2"}]}');
});

system.afterEvents.scriptEventReceive.subscribe((data) => {
    if (data.id != "mc:portal_black_teleport") return;

    const portalBlack = data.sourceEntity; // Quem ativou o evento (deve ser o portal preto)

    if (!portalBlack) {
        console.warn("O evento foi acionado, mas a entidade de origem é indefinida.");
        return;
    }

    const dimension = portalBlack.dimension;

    // Pegar o jogador mais próximo do portal preto
    const players = dimension.getPlayers({ location: portalBlack.location, maxDistance: 2 });
    const player = players[0];

    if (!player) {
        console.warn("Nenhum jogador encontrado próximo ao portal preto.");
        return;
    }

    const playerId = player.id;

    console.warn(`Evento acionado pelo portal preto. Encontrado jogador ${player.name} (ID: ${playerId}).`);

    // Procurar pela entidade do portal laranja que tem a tag associada ao playerId
    const ent = dimension.getEntities({ tags: [`portal_orange_${playerId}`] })[0];

    if (!ent) {
        console.warn(`Nenhum portal laranja encontrado para o jogador ${player.name}.`);
        return;
    }

    console.warn(`Portal laranja encontrado! Teletransportando ${player.name} para ${ent.location.x}, ${ent.location.y}, ${ent.location.z}.`);

    // Teletransporta o jogador para a localização do portal laranja
    player.teleport(ent.location);

    // Remove a tag temporária do portal laranja
    player.runCommandAsync('event entity @e[type=mc:hades_portal_orange] remove_tag_timer');

    // Cancela o teletransporte do portal laranja
    player.runCommandAsync('event entity @e[type=mc:hades_portal_orange] cancel_tp');

    // Envia uma mensagem na actionbar para todos os jogadores
    player.runCommandAsync('titleraw @a actionbar {"rawtext":[{"translate":"function.horb3"}]}');
});
