
import { world, system, ItemStack, ItemTypes, BlockPermutation } from "@minecraft/server";

const IDENTIFICADOR_DO_ITEM = "mc:fire_orb"; // Identificador do seu item
const vanillaCampfire = ["minecraft:campfire", "minecraft:soul_campfire"];
const vanillaCandle = ["minecraft:candle"];


world.beforeEvents.itemUseOn.subscribe((event) => {
    const { itemStack, block, source, player } = event;
    const blockLocation = block.location;
    const blockState = block.permutation.getState("minecraft:cardinal_direction");
    const blockCandle = block.permutation.getState("candles");

    // Verifica se o item usado é o correto
    if (itemStack.typeId !== IDENTIFICADOR_DO_ITEM) return;
    
        // Acende a fogueira apagada
        if (vanillaCampfire.includes(block.typeId)) {
            system.run(() => {
                if (block.permutation.matches("minecraft:campfire", { "extinguished": true, "minecraft:cardinal_direction": blockState })) {
                    block.setPermutation(BlockPermutation.resolve("minecraft:campfire", { "extinguished": false, "minecraft:cardinal_direction": blockState }));
                    // Som de acender a fogueira (som da bola de fogo do ghast)
                    block.dimension.playSound("mob.ghast.fireball", blockLocation, { pitch: 1.0, volume: 0.5 });
                    
                    // Consome o item apenas se estiver no modo sobrevivência
                    if (player.getGameMode() !== "creative") {
                        itemStack.amount--;
                    }
                }
                if (block.permutation.matches("minecraft:soul_campfire", { "extinguished": true, "minecraft:cardinal_direction": blockState })) {
                    block.setPermutation(BlockPermutation.resolve("minecraft:soul_campfire", { "extinguished": false, "minecraft:cardinal_direction": blockState }));
                    // Som de acender a fogueira (som da bola de fogo do ghast)
                    block.dimension.playSound("mob.ghast.fireball", blockLocation, { pitch: 1.0, volume: 0.5 });

                    // Consome o item apenas se estiver no modo sobrevivência
                    if (player.getGameMode() !== "creative") {
                        itemStack.amount--;
                    }
                }
            });
        }
        
                if (block.typeId.includes("candle")) {
    system.run(() => {
        if (block.permutation.matches(block.typeId, { "lit": false, "candles": blockCandle })) {
            block.setPermutation(BlockPermutation.resolve(block.typeId, { "lit": true, "candles": blockCandle }));
            block.dimension.playSound("mob.ghast.fireball", blockLocation, { pitch: 1.0, volume: 0.5 });

            if (player.getGameMode() !== "creative") {
                itemStack.amount--;
            }
        }
    });
}


        // Ativa TNT
        if (block.typeId === "minecraft:tnt") {
            const { x, y, z } = block.location;
            block.dimension.runCommandAsync(`setblock ${x} ${y} ${z} air`);
            block.dimension.runCommandAsync(`summon tnt ${x + 0.5} ${y} ${z + 0.5}`);
            block.dimension.playSound("mob.ghast.fireball", blockLocation, { pitch: 1.0, volume: 0.5 });
            block.dimension.playSound("random.fuse", blockLocation, { pitch: 1.0, volume: 0.5 });
        }
        
});

function getMobName(mobTypeId) {
    const name = mobTypeId.split(':')[1].replace(/_/g, ' '); // Remove o prefixo e substitui underscores por espaços
    return name
        .toLowerCase() // Converte toda a string para minúsculas
        .split(' ') // Divide a string em palavras
        .map(word => word.charAt(0).toUpperCase() + word.slice(1)) // Capitaliza a primeira letra de cada palavra
        .join(' '); // Junta as palavras de volta em uma string
}

world.beforeEvents.playerInteractWithEntity.subscribe(async eventData => {
    const entity = eventData.target;
    const player = eventData.player;
    const hand = player.getComponent("equippable").getEquipment("Mainhand");
    const lore = hand.getLore();
    system.run(() => {
        if (entity && entity.typeId != "minecraft:player" && hand.typeId == "mc:genie_lamp" && lore.length == 0) {
            const newItem = new ItemStack(ItemTypes.get("mc:genie_lamp_captured"));
            const mobName = getMobName(entity.typeId); // Usar a função para obter o nome do mob
            const test = entity.id;
            
            newItem.setLore(["§fID: " + test, "§fMob: " + mobName]);
            newItem.nameTag = `%item.lampc.name\n%item.lampc.desc\n${mobName}`
            player.runCommandAsync(`titleraw @p actionbar {"rawtext":[{"translate":"function.lampc"}]}`);
            entity.runCommandAsync(`structure save ${entity.id}_mob_capture_device ${entity.location.x} ${entity.location.y} ${entity.location.z} ${entity.location.x} ${entity.location.y} ${entity.location.z} true disk false`).then(r => {
                entity.remove();
                player.getComponent("equippable").setEquipment("Mainhand", newItem);
            });
        }
    });
});

let isCapturing = false;
world.beforeEvents.itemUseOn.subscribe(async ({ itemStack: item, source: player, block }) => {
    if (isCapturing) return;  // Evita execução múltipla
    isCapturing = true;
    
    if (item && item.getLore().length > 0 && item.typeId == "mc:genie_lamp_captured") {
        let newItem = new ItemStack(ItemTypes?.get("mc:genie_lamp"));
        player.runCommandAsync(`titleraw @p actionbar {"rawtext":[{"translate":"function.lampe"}]}`);
        
        await player.runCommandAsync(`structure load ${Number(item.getLore()[0].split("§fID: ")[1])}_mob_capture_device ${block.location.x} ${block.location.y + 2} ${block.location.z}`);
        newItem.setLore([]);
        player.getComponent("equippable").setEquipment("Mainhand", newItem);
    }

    isCapturing = false;
});
