import { world, EntityDamageCause, GameMode } from "@minecraft/server";

world.afterEvents.entityHitEntity.subscribe(eventData => {
  const damager = eventData.damagingEntity;
  const target = eventData.hitEntity;
  const { x, y, z } = damager.location;
  const damagerEquippable = damager.getComponent("equippable");
  if (!damagerEquippable) return;
  const itemUsed = damagerEquippable.getEquipment("Mainhand");
  const customWeapons = [
    "mc:ice_sword",
    "mc:fire_sword",
    "mc:rock_golem_fist",
    "mc:minotaur_axe"
  ]
  if (itemUsed === undefined) return;
  if (customWeapons.includes(itemUsed.typeId)) {
    if (itemUsed.typeId === "mc:ice_sword") {
      if (target.matches({ excludeFamilies:[ "golem", "elder_golem", "irongolem", "snowgolem" ] })) {
        const pitchRange = Math.random() * (1.2 - 0.8) + 0.8;
        target.applyDamage( 1, { cause: "override", damagingEntity: damager });
        target.addEffect("slowness", 300, {amplifier: 1, showParticles: true });
        damager.playSound("mob.bee.sting", { pitch: pitchRange, location: damager.location, volume: 0.5 });
      }
      const itemEnchantable = itemUsed.getComponent("enchantable");
      const unbreakingLevel = itemEnchantable?.getEnchantment("unbreaking")?.level ?? 0;
      const breakChance = 100 / (unbreakingLevel + 1);
      const randomizeChance = Math.random() * 100;
      if (breakChance < randomizeChance) return;
      const itemDurability = itemUsed.getComponent("durability");
      if (!itemDurability) return;
      const itemMaxDurability = itemDurability.maxDurability;
      const itemDurabilityDamage = itemDurability.damage;
      const durabilityCalculation = itemMaxDurability - itemDurabilityDamage;
      let damageAmount = 0;
      if (damager.typeId === "minecraft:player") {
        if (!damager.matches({ gameMode: GameMode.creative })) {
          damageAmount = 1;
        }
      else return;
      }
      else {
        damageAmount = 1;
      };
      const currentDurability = durabilityCalculation - damageAmount;
      if (currentDurability <= 0) {
        damager.playSound("random.break", { pitch: 0.9, location: damager.location, volume: 1 });
        damagerEquippable.setEquipment("Mainhand", null);
      }
      else {
        itemDurability.damage += damageAmount;
        damagerEquippable.setEquipment("Mainhand", itemUsed)
      }
    }
    if (itemUsed.typeId === "mc:fire_sword") {
      if (target.matches({ excludeFamilies:[ "golem", "elder_golem", "irongolem", "snowgolem" ] })) {
        const pitchRange = Math.random() * (1.2 - 0.8) + 0.8;
        target.applyDamage( 1, { cause: "override", damagingEntity: damager });
        damager.playSound("mob.bee.sting", { pitch: pitchRange, location: damager.location, volume: 0.5 });
      }
      else {
        target.setOnFire( 4, true );
      }
      const itemEnchantable = itemUsed.getComponent("enchantable");
      const unbreakingLevel = itemEnchantable?.getEnchantment("unbreaking")?.level ?? 0;
      const breakChance = 100 / (unbreakingLevel + 1);
      const randomizeChance = Math.random() * 100;
      if (breakChance < randomizeChance) return;
      const itemDurability = itemUsed.getComponent("durability");
      if (!itemDurability) return;
      const itemMaxDurability = itemDurability.maxDurability;
      const itemDurabilityDamage = itemDurability.damage;
      const durabilityCalculation = itemMaxDurability - itemDurabilityDamage;
      let damageAmount = 0;
      if (damager.typeId === "minecraft:player") {
        if (!damager.matches({ gameMode: GameMode.creative })) {
          damageAmount = 1;
        }
      else return;
      }
      else {
        damageAmount = 1;
      };
      const currentDurability = durabilityCalculation - damageAmount;
      if (currentDurability <= 0) {
        damager.playSound("random.break", { pitch: 0.9, location: damager.location, volume: 1 });
        damagerEquippable.setEquipment("Mainhand", null);
      }
      else {
        itemDurability.damage += damageAmount;
        damagerEquippable.setEquipment("Mainhand", itemUsed)
      }
    }
    if (itemUsed.typeId === "mc:minotaur_axe") {
      if (target.matches({ excludeFamilies:[ "golem", "elder_golem", "irongolem", "snowgolem" ] })) {
        const pitchRange = Math.random() * (1.2 - 0.8) + 0.8;
        target.applyDamage( 1, { cause: "override", damagingEntity: damager });
        damager.playSound("mob.bee.sting", { pitch: pitchRange, location: damager.location, volume: 0.5 });
      }
      const itemEnchantable = itemUsed.getComponent("enchantable");
      const unbreakingLevel = itemEnchantable?.getEnchantment("unbreaking")?.level ?? 0;
      const breakChance = 100 / (unbreakingLevel + 1);
      const randomizeChance = Math.random() * 100;
      if (breakChance < randomizeChance) return;
      const itemDurability = itemUsed.getComponent("durability");
      if (!itemDurability) return;
      const itemMaxDurability = itemDurability.maxDurability;
      const itemDurabilityDamage = itemDurability.damage;
      const durabilityCalculation = itemMaxDurability - itemDurabilityDamage;
      let damageAmount = 0;
      if (damager.typeId === "minecraft:player") {
        if (!damager.matches({ gameMode: GameMode.creative })) {
          damageAmount = 1;
        }
      else return;
      }
      else {
        damageAmount = 1;
      };
      const currentDurability = durabilityCalculation - damageAmount;
      if (currentDurability <= 0) {
        damager.playSound("random.break", { pitch: 0.9, location: damager.location, volume: 1 });
        damagerEquippable.setEquipment("Mainhand", null);
      }
      else {
        itemDurability.damage += damageAmount;
        damagerEquippable.setEquipment("Mainhand", itemUsed)
      }
    }
    if (itemUsed.typeId === "mc:rock_golem_fist") {
      if (target.matches({ excludeFamilies:[ "golem", "elder_golem", "irongolem", "snowgolem" ] })) {
        const pitchRange = Math.random() * (1.2 - 0.8) + 0.8;
        target.applyDamage( 1, { cause: "override", damagingEntity: damager });
        damager.playSound("mob.bee.sting", { pitch: pitchRange, location: damager.location, volume: 0.5 });
      }
      const itemEnchantable = itemUsed.getComponent("enchantable");
      const unbreakingLevel = itemEnchantable?.getEnchantment("unbreaking")?.level ?? 0;
      const breakChance = 100 / (unbreakingLevel + 1);
      const randomizeChance = Math.random() * 100;
      if (breakChance < randomizeChance) return;
      const itemDurability = itemUsed.getComponent("durability");
      if (!itemDurability) return;
      const itemMaxDurability = itemDurability.maxDurability;
      const itemDurabilityDamage = itemDurability.damage;
      const durabilityCalculation = itemMaxDurability - itemDurabilityDamage;
      let damageAmount = 0;
      if (damager.typeId === "minecraft:player") {
        if (!damager.matches({ gameMode: GameMode.creative })) {
          damageAmount = 1;
        }
      else return;
      }
      else {
        damageAmount = 1;
      };
      const currentDurability = durabilityCalculation - damageAmount;
      if (currentDurability <= 0) {
        damager.playSound("random.break", { pitch: 0.9, location: damager.location, volume: 1 });
        damagerEquippable.setEquipment("Mainhand", null);
      }
      else {
        itemDurability.damage += damageAmount;
        damagerEquippable.setEquipment("Mainhand", itemUsed)
      }
    }
    
  }
  else return;
});
