import { world, EquipmentSlot, system, Player } from "@minecraft/server";

// Detecta quando um jogador morre
world.afterEvents.entityDie.subscribe(event => {
    const entity = event.deadEntity;

    if (entity instanceof Player) {
        const player = entity;
        if (player.getGameMode() !== "creative") {
            const equipment = player.getComponent("minecraft:equippable");
            const headEquipment = equipment?.getEquipment(EquipmentSlot.Head);

            if (headEquipment && headEquipment.typeId === "mc:hades_mantle") {
                // Dar o item "mc:hades_portal_orb" ao jogador
                player.runCommandAsync(`gamerule sendcommandfeedback false`);
                player.runCommandAsync(`give @s mc:hades_portal_orb 1`);
                player.runCommandAsync(`gamerule sendcommandfeedback true`);

                // Criar um portal preto no local da morte do jogador
                const position = player.location;
                const portal = player.dimension.spawnEntity("mc:hades_portal_black", position);

                // Adicionar uma tag única ao portal preto
                portal.addTag(`portal_black_${player.id}`);
            }
        }
    }
});

world.beforeEvents.itemUseOn.subscribe(async ({ itemStack: item, source: player, block }) => {
    if (item && item.typeId == "mc:hades_portal_orb" && player.getGameMode() !== "creative") {
        const position = block.location;
        const playerId = player.id;
        const equipment = player.getComponent("equippable");
        const selectedItem = equipment.getEquipment(EquipmentSlot.Mainhand);

        // Mantendo o comando summon
        await player.runCommandAsync(`summon mc:hades_portal_orange ${position.x} ${position.y + 1} ${position.z}`);

        // Encontre a entidade criada
        const portalOrange = player.dimension.getEntities({ location: position, maxDistance: 10 })[0];

        if (portalOrange) {
            // Adiciona a tag ao portal
            portalOrange.addTag(`portal_orange_${playerId}`);
            console.warn(`Portal laranja criado em ${position.x}, ${position.y}, ${position.z}`);
        }

        // Remover o item se ele for único
        if (selectedItem.amount === 1) {
            equipment.setEquipment(EquipmentSlot.Mainhand, undefined);
        }
    }
});

function teleportPlayer(player, portal, messageKey) {
    console.warn(`Teletransportando ${player.name} para ${portal.location.x}, ${portal.location.y}, ${portal.location.z}.`);
    player.teleport(portal.location);

    // Usando titleraw para exibir a mensagem na ActionBar
    player.runCommandAsync(`titleraw @a actionbar {"rawtext":[{"translate":"${messageKey}"}]}`);
}

system.afterEvents.scriptEventReceive.subscribe((data) => {
    if (data.id !== "mc:portal_orange_teleport") return;

    const portalOrange = data.sourceEntity;
    if (!portalOrange) return console.warn("Evento acionado, mas a entidade de origem é indefinida.");

    const players = portalOrange.dimension.getPlayers({ location: portalOrange.location, maxDistance: 2 });
    const player = players[0];

    if (!player) return console.warn("Nenhum jogador encontrado próximo ao portal laranja.");

    const portalBlack = portalOrange.dimension.getEntities({ tags: [`portal_black_${player.id}`] })[0];
    if (!portalBlack) return console.warn(`Nenhum portal preto encontrado para ${player.name}.`);

    teleportPlayer(player, portalBlack, "function.horb2");

    // Substituindo `/event` por `triggerEvent`
    portalBlack.triggerEvent("remove_tag_timer");
});

system.afterEvents.scriptEventReceive.subscribe((data) => {
    if (data.id !== "mc:portal_black_teleport") return;

    const portalBlack = data.sourceEntity;
    if (!portalBlack) return console.warn("Evento acionado, mas a entidade de origem é indefinida.");

    const players = portalBlack.dimension.getPlayers({ location: portalBlack.location, maxDistance: 2 });
    const player = players[0];

    if (!player) return console.warn("Nenhum jogador encontrado próximo ao portal preto.");

    const portalOrange = portalBlack.dimension.getEntities({ tags: [`portal_orange_${player.id}`] })[0];
    if (!portalOrange) return console.warn(`Nenhum portal laranja encontrado para ${player.name}.`);

    teleportPlayer(player, portalOrange, "function.horb3");

    // Substituindo `/event` por `triggerEvent`
    portalOrange.triggerEvent("remove_tag_timer");
    portalOrange.triggerEvent("cancel_tp");
});
