import { world, system, ItemStack, BlockPermutation } from '@minecraft/server';
import { randomTickComponent, OnTickComponent, leafDrop } from "./OnTick"
/* import { treeInteractComponent, leafDefaultComponent } from './tree'; 
desativado pois não sera usado no momento */
import { FenceInteractComponent, PlaceComponent, PlaceGateComponent, GateInteract } from './fence';
import { slabComponent } from './slabs';
import './door'
import './trapdoor'
import './break_leaves'
import './fallingLeaves'

// Add more mappings here as needed - Note, the doors need to be set on door.js at line 237.
const isSlab = new Set([
  "mc:leshy_slab",
  "mc:leshy_slab_item",
  "mc:grassy_leshy_slab",
  "mc:grassy_leshy_slab_item"
]);
const fenceSet = new Set([
  "mc:leshy_fence",
  "mc:leshy_fence_gate",
  "mc:grassy_leshy_fence",
  "mc:grassy_leshy_fence_gate"
]);
export const gateSet = new Set(
  Array.from(fenceSet).filter(item => item.includes("_gate"))
);
const fenceMap = new Map([
  ["mc:leshy_fence", "mc:leshy_fence"],
  ["mc:leshy_fence_inventory", "mc:leshy_fence"],
  ["mc:leshy_fence_inventory_item", "mc:leshy_fence"],
  ["mc:grassy_leshy_fence", "mc:grassy_leshy_fence"],
  ["mc:grassy_leshy_fence_inventory", "mc:grassy_leshy_fence"],
  ["mc:grassy_leshy_fence_inventory_item", "mc:grassy_leshy_fence"]
]);
const leafSet = new Set([
  "wiki:white_oak_leaves",
  "wiki:orange_tree_leaves"
]);
const fruitSet = new Set([
  "wiki:orange_tree_leaves"
]);
const leafDropMap = new Map([
  ["wiki:orange_tree_leaves", "wiki:orange_item"],
  ["wiki:white_oak_leaves", "wiki:custom_sapling_placer"]
]);
const saplingBlocks = [
  {
    blockID: "wiki:white_oak_sapling",
    maxStage: 2,
    growChance: {
      numerator: 1,
      denominator: 3
    }
  },
  {
    blockID: "wiki:orange_tree_sapling",
    maxStage: 2,
    growChance: {
      numerator: 1,
      denominator: 3
    }
  },
  {
    blockID: "wiki:orange_tree_leaves",
    maxStage: 3,
    growChance: {
      numerator: 1,
      denominator: 7 // adjust this to adjust speed of fruit growth
    }
  }
  // Add other sapling types here
];
const logSet = new Set([
  "wiki:white_oak_log",
  "wiki:orange_tree_log"
]);
const sapSet = new Set([
  "wiki:white_oak_sapling",
  "wiki:orange_tree_sapling"
]);
const logMap = new Map([
  ["wiki:orange_tree_log", "wiki:stripped_orange_tree_log"],
  ["wiki:white_oak_log", "wiki:stripped_white_oak_log"]
]);
const trapSet = new Set([
  "mc:leshy_trapdoor",
  "mc:grassy_leshy_trapdoor"
]);

// This handles the durality damage for all blocks (except slabs - handled in slabs.js) and removing the block above for fences and gates
const DestroyComponent = {
  onPlayerDestroy(e) {
    const { block, player } = e;
    const aboveBlock = block.above();
    const blockBelow = block.below();

    // Remove wiki:fence_inventory or invisible fence gate on top if present
    if ((aboveBlock.typeId.includes("_inventory")) || (fenceSet.has(aboveBlock.typeId) && aboveBlock.permutation.getState('mc:invisible'))) {
      aboveBlock.setType("minecraft:air");
    }

    // add wiki:fence_inventory if fence present below
    if (fenceSet.has(blockBelow.typeId)) {
      block.setPermutation(
        BlockPermutation.resolve("mc:grassy_leshy_fence_inventory", { "mc:post": 1 })
      );
    }
    
  },
};

const customComponents = {
  "mc:fence_interact": FenceInteractComponent,
  "mc:on_player_destroy": DestroyComponent,
  "mc:on_tick": OnTickComponent,
  "mc:on_player_placed": PlaceComponent,
//  "mc:random_tick": randomTickComponent, //
   "mc:slab_interact": slabComponent,
 // "mc:interact_tree": treeInteractComponent, //
  "mc:gate_interact": GateInteract,
  "mc:gate_placed": PlaceGateComponent/*,
  "mc:leaf_place": leafDefaultComponent */
};

let lastBrokenBlockType = null;
let lastBrokenBlockIsDouble = false;

// Use this world initialization event to register all custom components
world.beforeEvents.worldInitialize.subscribe(({ blockComponentRegistry }) => {
  for (const [componentName, componentImplementation] of Object.entries(customComponents)) {
    blockComponentRegistry.registerCustomComponent(componentName, componentImplementation);
  }
});

world.beforeEvents.playerBreakBlock.subscribe((eventData) => {
  const { block, player, dimension } = eventData;
  const blockBelow = block.below();
  const inventory = player.getComponent("inventory");
  const selectedItem = inventory.container.getItem(player.selectedSlotIndex);
  const { hasSilkTouch } = getRelevantEnchantments(selectedItem);

  if (selectedItem && leafSet.has(block.typeId) && selectedItem.typeId ===  "minecraft:shears" && !hasSilkTouch) {
    const spawnItem = block.typeId;
    system.runTimeout(() => {
      block.dimension.spawnItem(new ItemStack(spawnItem, 1), block.location);
    }, 1);
  } else if(leafSet.has(block.typeId) && !hasSilkTouch){
    leafDrop(block, true);
  }

  if (isSlab.has(block.typeId)) {
    lastBrokenBlockType = block.typeId + "_item";
    lastBrokenBlockIsDouble =
      block.permutation.getState("mc:double") === true;
  system.runTimeout(() => {
    if (!selectedItem) {
      slabDrop(block, dimension, player);
      
    } else {
      slabDrop(block, dimension, player);
    }
  }, 1);
} else if (fenceSet.has(blockBelow.typeId)) {
  system.runTimeout(() => {
      if (blockBelow.typeId.includes("mc:") && blockBelow.typeId.includes("_gate")) {
        const currentStates = blockBelow.permutation.getAllStates();

        const cardinalDirection = currentStates['minecraft:cardinal_direction'];
            const newAboveStates = { ...currentStates };
            if (cardinalDirection === 'south') {
                delete newAboveStates['minecraft:cardinal_direction'];
            }
            newAboveStates['mc:invisible'] = true;
            block.setPermutation(BlockPermutation.resolve(blockBelow.typeId, newAboveStates));
          
      } else if (blockBelow.typeId.includes("mc:") && !blockBelow.typeId.includes("_gate")) {
        block.setPermutation(
          BlockPermutation.resolve("mc:grassy_leshy_fence_inventory", { "mc:post": 1 })
        );
      }
  }, 1);
  } 
});

function slabDrop(block, dimension, player) {
  if (player.getGameMode() !== "creative") {
    if (lastBrokenBlockType) {
      const blockData = lastBrokenBlockType;
  
      if (blockData) {
        let dropItem;
  
        if (
          lastBrokenBlockIsDouble
        ) {
          dropItem = new ItemStack(lastBrokenBlockType, 1);
        } 
        if (dropItem) {
          dimension.spawnItem(dropItem, block.location);
        }
      }
      lastBrokenBlockType = null;
      lastBrokenBlockIsDouble = false;
    }
  }
}


function getRelevantEnchantments(item) {
  let unbreakingLevel = 0;
  let hasSilkTouch = false;

  try {
      const enchantableComponent = item.getComponent("minecraft:enchantable");
      if (enchantableComponent) {
          const enchantments = enchantableComponent.getEnchantments();
          for (const enchant of enchantments) {
              if (enchant.type.id === "unbreaking") {
                  unbreakingLevel = enchant.level;
              } else if (enchant.type.id === "silk_touch") {
                  hasSilkTouch = true;
              }
          }
      }
  } catch (error) {
  }
  return { unbreakingLevel, hasSilkTouch };
}



export {
  getRelevantEnchantments,
  isSlab,
  fenceSet,
  fenceMap,
  leafSet,
  leafDropMap,
  saplingBlocks,
  logSet,
  sapSet,
  logMap,
  trapSet,
  fruitSet
}