import { world, ItemStack, system, BlockPermutation } from "@minecraft/server";
import { isSlab } from './config';

export const slabComponent = {
  onPlayerInteract(e) {
    const { block, player, face } = e;
    const equipment = player.getComponent("equippable");
    const selectedItem = equipment.getEquipment("Mainhand");
    if (!selectedItem) return;

    // Obtém estados do bloco e direção com base no jogador
    const { stateName, stateValue } = getBlockStatesAndDirection(player, selectedItem, face);

    // Se o jogador estiver usando um balde de água
    if (selectedItem.typeId === "minecraft:water_bucket") {
    	if (e.dimension.id !== "minecraft:nether") {
      // Alaga o bloco se ele não estiver alagado
      if (!block.isWaterlogged) {
        block.setWaterlogged(true); // Torna o bloco alagado
      }
      player.playSound( "bucket.empty_water" );
      if (player.getGameMode() !== "creative") {
            equipment.setEquipment( "Mainhand", new ItemStack("minecraft:bucket", 1) );
          }
          } else {
          e.dimension.spawnParticle("minecraft:water_evaporation_bucket_emitter", block.center());
          player.playSound("random.fizz");
          // If not in creative mode, replace water bucket with empty bucket
          if (player.getGameMode() !== "creative") {
            equipment.setEquipment( "Mainhand", new ItemStack("minecraft:bucket", 1) );
          }
        }
          
      return;
    }
    
    if (selectedItem.typeId === "minecraft:bucket") {
      // Deixa o bloco  não alagado
      if (block.isWaterlogged) {
        block.setWaterlogged(false); // Torna o bloco não alagado
      }
      player.playSound( "bucket.fill_water" );
      if (player.getGameMode() !== "creative") {
            if (selectedItem.amount === 1) {
                equipment.setEquipment("Mainhand", undefined);
              } else {
                selectedItem.amount -= 1;
                equipment.setEquipment("Mainhand", selectedItem);
              }
            player.runCommandAsync(`gamerule sendcommandfeedback false`);
            player.runCommandAsync(`give @s minecraft:water_bucket 1`);
            player.runCommandAsync(`gamerule sendcommandfeedback true`);
          }
      return;
    }

    // Obtém a posição do bloco adjacente com base na face clicada
    let adjacentBlockPos;
    switch (face) {
      case "North":
        adjacentBlockPos = { x: block.x, y: block.y, z: block.z - 1 };
        break;
      case "South":
        adjacentBlockPos = { x: block.x, y: block.y, z: block.z + 1 };
        break;
      case "East":
        adjacentBlockPos = { x: block.x + 1, y: block.y, z: block.z };
        break;
      case "West":
        adjacentBlockPos = { x: block.x - 1, y: block.y, z: block.z };
        break;
      case "Up":
        adjacentBlockPos = { x: block.x, y: block.y + 1, z: block.z };
        break;
      case "Down":
        adjacentBlockPos = { x: block.x, y: block.y - 1, z: block.z };
        break;
    }

    const adjacentBlock = e.dimension.getBlock(adjacentBlockPos);

    // Se o bloco adjacente é "air", "water" ou "lava", procede com a colocação
    if (["minecraft:air", "minecraft:water", "minecraft:lava"].includes(adjacentBlock.typeId)) {
      const verticalHalf = block.permutation.getState("minecraft:vertical_half");

      // Se o item selecionado for uma slab personalizada
      if (isSlab.has(selectedItem.typeId)) {
        // Determina se deve criar uma slab dupla ou colocar uma nova slab
        const isBottomUp = verticalHalf === "bottom" && face === "Up";
        const isTopDown = verticalHalf === "top" && face === "Down";

        if ((isBottomUp || isTopDown) && selectedItem.typeId.replace("_item", "") === block.typeId) {
          // Cria uma slab dupla
          if (!block.permutation.getState("mc:double")) {
            // Se o bloco estiver waterlogged, remove-o temporariamente para que o vanilla atualize
            
            // Define a slab como dupla via permutação
            block.setPermutation(block.permutation.withState("mc:double", true));
            // Remove o waterlogging do bloco duplo utilizando o método nativo
            block.setWaterlogged(false);

            player.playSound("use.wood");
            if (player.getGameMode() !== "creative") {
              if (selectedItem.amount === 1) {
                equipment.setEquipment("Mainhand", undefined);
              } else {
                selectedItem.amount -= 1;
                equipment.setEquipment("Mainhand", selectedItem);
              }
            }
          } else {
            // Se já for slab dupla, trata a colocação normalmente
            placeSlab(
              player,
              selectedItem,
              adjacentBlockPos,
              face,
              equipment,
              block,
              true
            );
          }
        } else if (block.permutation.getState("mc:double")) {
          // Trata a colocação contra o lado de uma slab dupla
          placeSlab(
            player,
            selectedItem,
            adjacentBlockPos,
            face,
            equipment,
            block,
            true
          );
        } else {
          // Coloca uma slab em cima, embaixo ou lateral de uma slab simples
          placeSlab(
            player,
            selectedItem,
            adjacentBlockPos,
            face,
            equipment,
            block,
            false
          );
        }
      } else {
        // Trata a colocação de blocos que não são slabs com estados
        const blockTypeId = selectedItem.typeId.endsWith("_item")
          ? selectedItem.typeId.replace("_item", "")
          : selectedItem.typeId;

        const commandBlock = stateName
          ? `setblock ${adjacentBlockPos.x} ${adjacentBlockPos.y} ${adjacentBlockPos.z} ${blockTypeId} ["${stateName}"="${stateValue}"]`
          : `setblock ${adjacentBlockPos.x} ${adjacentBlockPos.y} ${adjacentBlockPos.z} ${blockTypeId}`;

        system.runTimeout(() => {
          try {
            player.runCommand(commandBlock);
            if (player.getGameMode() !== "creative") {
              if (selectedItem.amount === 1) {
                equipment.setEquipment("Mainhand", undefined);
              } else {
                selectedItem.amount -= 1;
                equipment.setEquipment("Mainhand", selectedItem);
              }
            }
          } catch (error) {
            console.warn(`Error: ${error.message}`);
          }
        }, 1);
      }
    }
  },
};

export function placeSlab(
  player,
  selectedItem,
  adjacentBlockPos,
  face,
  equipment,
  targetBlock,
  isDoubleSlab
) {
  let verticalHalf;
  const raycastOptions = {
    maxDistance: 5,
    includePassableBlocks: false,
  };

  const raycastHit = player.getBlockFromViewDirection(raycastOptions);
  if (raycastHit && isDoubleSlab) {
    const faceLocation = raycastHit.faceLocation;
    verticalHalf = (face === "Up" || face === "Down")
      ? (face === "Up" ? "bottom" : "top")
      : (faceLocation.y > 0.5 ? "top" : "bottom");
  } else {
    if (face === "Up") {
      verticalHalf = "bottom";
    } else if (face === "Down") {
      verticalHalf = "top";
    } else {
      verticalHalf = isSlab.has(targetBlock.typeId)
        ? targetBlock.permutation.getState("minecraft:vertical_half")
        : "bottom";
    }
  }

  const blockTypeId = selectedItem.typeId.endsWith("_item")
    ? selectedItem.typeId.replace("_item", "")
    : selectedItem.typeId;

  const command = `setblock ${adjacentBlockPos.x} ${adjacentBlockPos.y} ${adjacentBlockPos.z} ${blockTypeId} ["minecraft:vertical_half"="${verticalHalf}"]`;

  system.runTimeout(() => {
    try {
      player.runCommand(command);
      if (player.getGameMode() !== "creative") {
        if (selectedItem.amount === 1) {
          equipment.setEquipment("Mainhand", undefined);
        } else {
          selectedItem.amount -= 1;
          equipment.setEquipment("Mainhand", selectedItem);
        }
      }
    } catch (error) {
      console.warn(`Error: ${error.message}`);
    }
  }, 1);
}

function getBlockStatesAndDirection(player, selectedItem, face) {
  const blockType = selectedItem.typeId.replace("_item", "");
  let blockPermutation;
  try {
    blockPermutation = BlockPermutation.resolve(blockType);
  } catch (error) {
    console.warn(`Não foi possível resolver o bloco: ${blockType}. Erro: ${error.message}`);
    return { stateName: null, stateValue: null };
  }
  const blockStates = blockPermutation.getAllStates();
  let stateName = null;
  let stateValue = null;

  const statePriority = [
    "minecraft:block_face",
    "minecraft:cardinal_direction",
    "pillar_axis",
  ];
  for (const state of statePriority) {
    if (blockStates[state] !== undefined) {
      stateName = state;
      if (state === "pillar_axis") {
        stateValue = ["North", "South"].includes(face)
          ? "z"
          : ["East", "West"].includes(face)
          ? "x"
          : "y";
      } else if (state === "minecraft:cardinal_direction") {
        const rotation = player.getRotation();
        const rad = (rotation.y * Math.PI) / 180;
        const directionX = -Math.sin(rad);
        const directionZ = Math.cos(rad);
        stateValue = Math.abs(directionX) > Math.abs(directionZ)
          ? (directionX > 0 ? "east" : "west")
          : (directionZ > 0 ? "south" : "north");
      } else if (state === "minecraft:block_face") {
        stateValue = face.toLowerCase();
      } else {
        stateValue = blockStates[state];
      }
      break;
    }
  }
  return { stateName, stateValue };
}
