// Import necessary modules from Minecraft server API
import { world, BlockPermutation, ItemStack } from '@minecraft/server';
import { trapSet } from './config';

// Subscribe to the 'worldInitialize' event to register custom components
world.beforeEvents.worldInitialize.subscribe(eventData => {
    // Register a custom component named wiki:on_interact for trapdoor interaction
    eventData.blockComponentRegistry.registerCustomComponent('mc:TD_interact', {
    	onTick: (e) => {
    const { block, dimension } = e;
        // Quando a porta for quebrada no modo sobrevivência
function toggleTrapdoor(block, dimension) {
    const currentState = block.permutation.getState('mc:open');

    // Determina o novo estado do bloco (abre ou fecha a porta)
    const newOpenState = !currentState;

    // Resolve a nova permutação de bloco com o estado atualizado
    const newPermutation = BlockPermutation.resolve(block.typeId, {
        ...block.permutation.getAllStates(),
        'mc:open': newOpenState
    });

    // Atualiza a permutação do bloco
    block.setPermutation(newPermutation);

    // Define o som baseado no estado
    const sound = currentState ? 'open.wooden_trapdoor' : 'close.wooden_trapdoor';

    // Toca o som no bloco
    dimension.runCommand(`playsound ${sound} @a[x=${block.location.x},y=${block.location.y},z=${block.location.z}]`);
}
        // Verificação de estado da porta
        const isOpen = block.permutation.getState("mc:open");
        const isPowered = block.permutation.getState("mc:powered");
        const hasRedstone = block.getRedstonePower() > 0;

        if (isPowered && !hasRedstone) {
            block.setPermutation(block.permutation.withState("mc:powered", false));
            if (isOpen) {
                toggleTrapdoor(block, dimension);
            }
        } else if (!isPowered && hasRedstone) {
            block.setPermutation(block.permutation.withState("mc:powered", true));
            if (!isOpen) {
                toggleTrapdoor(block, dimension);
            }
        }
    },
        // Define the behavior when a player interacts with the trapdoor block
        onPlayerInteract(e) {
            // Destructure event data for easier access
            const { block, player } = e;

            // Get the equipment component for the player
            const equipment = player.getComponent('equippable');

            // Get the selected item from the player's mainhand
            const selectedItem = equipment.getEquipment('Mainhand');

            // Get the current state of the 'wiki:open' block trait
            const currentState = block.permutation.getState('mc:open');

            // Determine the new state of the 'wiki:open' block trait (toggle between true and false)
            const newOpenState = !currentState;

            // Resolve the new block permutation based on the current block type and updated states
            const newPermutation = BlockPermutation.resolve(block.typeId, {
                ...block.permutation.getAllStates(),
                'mc:open': newOpenState
            });

            // Set the block permutation to the newly resolved permutation
            block.setPermutation(newPermutation);

            // Determine the sound effect to play based on the current state of the trapdoor
            const sound = currentState ? 'open.wooden_trapdoor' : 'close.wooden_trapdoor';

            // Play the corresponding sound effect for opening or closing the trapdoor
            player.playSound(sound);

            // Check if the current dimension is not the Nether
        if (e.dimension.id !== "minecraft:nether") {
            // Check if the selected item is a water bucket
            if (selectedItem?.typeId === 'minecraft:water_bucket') {
                // Play sound effect
                player.playSound( "bucket.empty_water" );
                // If not in creative mode, replace water bucket with empty bucket
                if (player.getGameMode() !== "creative") {
                    equipment.setEquipment('Mainhand', new ItemStack('minecraft:bucket', 1));
                }
            }
            // Check if the block interacted is a wiki:trapdoor and the player is using a water bucket
            if (selectedItem?.typeId === 'minecraft:water_bucket') {
                // Save the current block states
                const currentStates = block.permutation.getAllStates();

                const verticalHalf = block.permutation.getState(
                    "minecraft:vertical_half"
                  );

                // Place the structure
                if (!block.isWaterlogged) {
        block.setWaterlogged(true); // Torna o bloco alagado
      }

                // Reapply the old block states to the new block
                const newStates = { ...newBlock.permutation.getAllStates(), ...currentStates };
                const newPermutation = BlockPermutation.resolve(newBlock.typeId, newStates);
                newBlock.setPermutation(newPermutation);
            }
            // Check if the selected item is a empty bucket and handle un-waterlogging
            else if ( selectedItem?.typeId === "minecraft:bucket" ) {

                if (block.permutation.getState("mc:waterlogged") ) {
        
                // Save the state of the block
                const currentPermutation = block.permutation;
                const currentBlock = block.typeId;
        
                player.playSound( "bucket.fill_water" );
                if (block.isWaterlogged) {
        block.setWaterlogged(false); // Torna o bloco alagado
      }
        
                // If not in creative mode, replace empty bucket with water bucket
                if (player.getGameMode() !== "creative") {
            if (selectedItem.amount === 1) {
                equipment.setEquipment("Mainhand", undefined);
              } else {
                selectedItem.amount -= 1;
                equipment.setEquipment("Mainhand", selectedItem);
              }
            player.runCommandAsync(`gamerule sendcommandfeedback false`);
            player.runCommandAsync(`give @s minecraft:water_bucket 1`);
            player.runCommandAsync(`gamerule sendcommandfeedback true`);
          }
                
                // Apply the new permutation
                block.setType(currentBlock);
        
                // Apply the previous permutation (now un waterlogged)
                const newPermutation = currentPermutation
                    .withState("mc:waterlogged", false)
                    .withState("minecraft:vertical_half", currentPermutation.getState("minecraft:vertical_half"));
                block.setPermutation(newPermutation);       
                }
            }
        } else if (trapSet.has(block.typeId) && selectedItem?.typeId === 'minecraft:water_bucket') {
                e.dimension.spawnParticle("minecraft:water_evaporation_bucket_emitter", block.center());
                player.playSound("random.fizz");
                // If not in creative mode, replace water bucket with empty bucket
                if (player.getGameMode() !== "creative") {
                  equipment.setEquipment( "Mainhand", new ItemStack("minecraft:bucket", 1) );
                }
              }
        }
    });
});