import { world, EquipmentSlot, ItemStack, system, Player } from "@minecraft/server";

// Detecta quando um jogador morre
world.afterEvents.entityDie.subscribe(event => {
    const entity = event.deadEntity;

    // Garante que a entidade seja um jogador
    if (entity instanceof Player) {
        const player = entity; // Agora temos certeza que é um Player

        if (player.getGameMode() !== "creative") { // Verifica se o jogador não está no Criativo
            const equipment = player.getComponent("minecraft:equippable");
            const headEquipment = equipment?.getEquipment(EquipmentSlot.Head);

            if (headEquipment && headEquipment.typeId === "mc:hades_mantle") {
                // Dar o item "mc:hades_portal_orb" ao jogador
                player.runCommand(`give "${player.nameTag}" mc:hades_portal_orb`);

                // Criar o portal preto no local da morte do jogador
                const position = player.location;
                const portal = player.dimension.spawnEntity("mc:hades_portal_black", position);

                // Adicionar uma tag ao portal para identificar que pertence ao jogador
                const playerTag = player.nameTag.replace(/[^a-zA-Z0-9]/g, '');
                portal.addTag(`portal_do_${playerTag}`);
            }
        }
    }
});

// Detecta quando um jogador usa o mc:hades_portal_orb
world.afterEvents.itemUse.subscribe(event => {
    const { source, itemStack } = event;

    // Garante que a entidade que usou o item é um jogador
    if (source instanceof Player) {
        const player = source; // Agora temos certeza que é um Player

        if (itemStack.typeId === "mc:hades_portal_orb" && player.getGameMode() !== "creative") {
            const playerTag = player.nameTag.replace(/[^a-zA-Z0-9]/g, '');
            const position = player.location;

            // Criar o portal laranja no local do jogador
            const portalOrange = player.dimension.spawnEntity("mc:hades_portal_orange", position);
            portalOrange.addTag(`portal_laranja_${playerTag}`);

            // Verificar se o portal preto do jogador existe
            system.runTimeout(() => {
                const portals = player.dimension.getEntities({ type: "mc:hades_portal_black", tags: [`portal_do_${playerTag}`] });

                if (portals.length > 0) {
                    const blackPortal = portals[0];
                    const destination = blackPortal.location;

                    // Teletransportar o jogador para o portal preto
                    player.teleport(destination, { dimension: player.dimension, checkForBlocks: false });

                    // Remover os dois portais
                    blackPortal.remove();
                    portalOrange.remove();
                }
            }, 20); // Pequeno delay para garantir que os portais tenham sido criados antes da verificação
        }
    }
});
